@@ -24,7 +24,9 @@ def modify_complex_key(name, amt):
     return modify_key_bytes(key_as_bytes(name), amt)
 
 
-control_codes = {}
+control_codes = {
+    defines.GLFW_KEY_BACKSPACE: b'\x08'
+}
 smkx_key_map = {}
 alt_codes = {
     defines.GLFW_KEY_TAB: b'\033\t',
@@ -47,6 +49,9 @@ SHIFTED_KEYS = {
     defines.GLFW_KEY_DOWN: key_as_bytes('kind'),
 }
 
+control_alt_codes = {
+    defines.GLFW_KEY_SPACE: b'\x1b\0',
+}
 
 for kf, kn in {
     defines.GLFW_KEY_UP: 'kcuu1',
@@ -64,6 +69,7 @@ for kf, kn in {
     alt_codes[kf] = modify_complex_key(kn, 3)
     shift_alt_codes[kf] = modify_complex_key(kn, 4)
     control_codes[kf] = modify_complex_key(kn, 5)
+    control_alt_codes[kf] = modify_complex_key(kn, 7)
 for f in range(1, 13):
     kf = getattr(defines, 'GLFW_KEY_F{}'.format(f))
     kn = 'kf{}'.format(f)
@@ -72,6 +78,7 @@ for f in range(1, 13):
     alt_codes[kf] = modify_complex_key(kn, 3)
     shift_alt_codes[kf] = modify_complex_key(kn, 4)
     control_codes[kf] = modify_complex_key(kn, 5)
+    control_alt_codes[kf] = modify_complex_key(kn, 7)
 for f in range(13, 26):
     kf = getattr(defines, 'GLFW_KEY_F{}'.format(f))
     kn = 'kf{}'.format(f)
@@ -211,6 +218,7 @@ ASCII_C0_SHIFTED = {
 }
 CTRL_SHIFT_KEYS = {getattr(defines, 'GLFW_KEY_' + k): v for k, v in ASCII_C0_SHIFTED.items()}
 CTRL_ALT_KEYS = {getattr(defines, 'GLFW_KEY_' + k) for k in string.ascii_uppercase}
+all_control_alt_keys = set(CTRL_ALT_KEYS) | set(control_alt_codes)
 
 
 def key_to_bytes(key, smkx, extended, mods, action):
@@ -229,8 +237,11 @@ def key_to_bytes(key, smkx, extended, mods, action):
             m = UN_SHIFTED_PRINTABLE if mods == defines.GLFW_MOD_ALT else SHIFTED_PRINTABLE
             data.append(0o33)
             data.extend(m[key])
-    elif mods == ctrl_alt_mod and key in CTRL_ALT_KEYS:
-        data.append(0x1b), data.extend(control_codes[key])
+    elif mods == ctrl_alt_mod and key in all_control_alt_keys:
+        if key in CTRL_ALT_KEYS:
+            data.append(0x1b), data.extend(control_codes[key])
+        else:
+            data.extend(control_alt_codes[key])
     else:
         key_map = cursor_key_mode_map[smkx]
         x = key_map.get(key)
@@ -261,7 +272,7 @@ def shortcut_matches(s, mods, key, scancode):
 
 
 def generate_key_table():
-    # To run this, use: python3 . -c "from kitty.keys import *; generate_key_table()"
+    # To run this, use: python3 . +runpy "from kitty.keys import *; generate_key_table()"
     import os
     from functools import partial
     f = open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'keys.h'), 'w')
