@@ -12,10 +12,10 @@ from .config import build_ansi_color_table
 from .constants import WindowGeometry, appname, get_boss, is_macos, is_wayland
 from .fast_data_types import (
     DECAWM, Screen, add_tab, glfw_post_empty_event, mark_tab_bar_dirty,
-    remove_tab, remove_window, set_active_tab, set_tab_bar_render_data,
-    swap_tabs, viewport_for_window, x11_window_id
+    next_window_id, pt_to_px, remove_tab, remove_window, set_active_tab,
+    set_tab_bar_render_data, swap_tabs, viewport_for_window, x11_window_id
 )
-from .layout import Rect, all_layouts
+from .layout import Rect, create_layout_object_for, evict_cached_layouts
 from .session import resolved_shell
 from .utils import color_as_int, log_error
 from .window import Window, calculate_gl_geometry
@@ -38,6 +38,8 @@ class Tab:  # {{{
         if not self.id:
             raise Exception('No OS window with id {} found, or tab counter has wrapped'.format(self.os_window_id))
         self.opts, self.args = tab_manager.opts, tab_manager.args
+        self.margin_width, self.padding_width = map(pt_to_px, (
+            self.opts.window_margin_width, self.opts.window_padding_width))
         self.name = getattr(session_tab, 'name', '')
         self.enabled_layouts = [x.lower() for x in getattr(session_tab, 'enabled_layouts', None) or self.opts.enabled_layouts]
         self.borders = Borders(self.os_window_id, self.id, self.opts)
@@ -137,14 +139,16 @@ class Tab:  # {{{
             if w is not None:
                 w.change_titlebar_color()
 
-    def create_layout_object(self, idx):
-        return all_layouts[idx](self.os_window_id, self.id, self.opts, self.borders.border_width)
+    def create_layout_object(self, name):
+        return create_layout_object_for(name, self.os_window_id, self.id, self.margin_width, self.padding_width, self.borders.border_width)
 
     def next_layout(self):
         if len(self.enabled_layouts) > 1:
-            try:
-                idx = self.enabled_layouts.index(self.current_layout.name)
-            except Exception:
+            for i, layout_name in enumerate(self.enabled_layouts):
+                if layout_name == self.current_layout.full_name:
+                    idx = i
+                    break
+            else:
                 idx = -1
             nl = self.enabled_layouts[(idx + 1) % len(self.enabled_layouts)]
             self.current_layout = self.create_layout_object(nl)
@@ -158,6 +162,17 @@ class Tab:  # {{{
         self.current_layout = self.create_layout_object(layout_name)
         self.relayout()
 
+    def resize_window_by(self, window_id, increment, is_horizontal):
+        increment_as_percent = self.current_layout.bias_increment_for_cell(is_horizontal) * increment
+        if self.current_layout.modify_size_of_window(self.windows, window_id, increment_as_percent, is_horizontal):
+            self.relayout()
+            return
+        return 'Could not resize'
+
+    def reset_window_sizes(self):
+        if self.current_layout.remove_all_biases():
+            self.relayout()
+
     def launch_child(self, use_shell=False, cmd=None, stdin=None, cwd_from=None, cwd=None, env=None):
         if cmd is None:
             if use_shell:
@@ -167,6 +182,7 @@ class Tab:  # {{{
         fenv = {}
         if env:
             fenv.update(env)
+        fenv['KITTY_WINDOW_ID'] = str(next_window_id())
         if not is_macos and not is_wayland:
             try:
                 fenv['WINDOWID'] = str(x11_window_id(self.os_window_id))
@@ -274,6 +290,7 @@ class Tab:  # {{{
         return window in self.windows
 
     def destroy(self):
+        evict_cached_layouts(self.id)
         for w in self.windows:
             w.destroy()
         self.windows = deque()
